package org.apache.bcel.classfile;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.io.DataInput;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Arrays;
import org.apache.bcel.classfile.Attribute;
import org.apache.bcel.classfile.CodeException;
import org.apache.bcel.classfile.ConstantPool;
import org.apache.bcel.classfile.LineNumberTable;
import org.apache.bcel.classfile.LocalVariableTable;
import org.apache.bcel.classfile.Utility;
import org.apache.bcel.classfile.Visitor;
import org.apache.commons.lang3.ArrayUtils;

public final class Code extends Attribute {
  private int maxStack;
  
  private int maxLocals;
  
  private byte[] code;
  
  private CodeException[] exceptionTable;
  
  private Attribute[] attributes;
  
  public Code(org.apache.bcel.classfile.Code c) {
    this(c.getNameIndex(), c.getLength(), c.getMaxStack(), c.getMaxLocals(), c.getCode(), c.getExceptionTable(), c.getAttributes(), c.getConstantPool());
  }
  
  Code(int nameIndex, int length, DataInput file, ConstantPool constantPool) throws IOException {
    this(nameIndex, length, file.readUnsignedShort(), file.readUnsignedShort(), (byte[])null, (CodeException[])null, (Attribute[])null, constantPool);
    int codeLength = file.readInt();
    this.code = new byte[codeLength];
    file.readFully(this.code);
    int exceptionTableLength = file.readUnsignedShort();
    this.exceptionTable = new CodeException[exceptionTableLength];
    for (int i = 0; i < exceptionTableLength; i++)
      this.exceptionTable[i] = new CodeException(file); 
    int attributesCount = file.readUnsignedShort();
    this.attributes = new Attribute[attributesCount];
    for (int j = 0; j < attributesCount; j++)
      this.attributes[j] = Attribute.readAttribute(file, constantPool); 
    setLength(length);
  }
  
  public Code(int nameIndex, int length, int maxStack, int maxLocals, byte[] code, CodeException[] exceptionTable, Attribute[] attributes, ConstantPool constantPool) {
    super((byte)2, nameIndex, length, constantPool);
    this.maxStack = maxStack;
    this.maxLocals = maxLocals;
    this.code = (code != null) ? code : ArrayUtils.EMPTY_BYTE_ARRAY;
    this.exceptionTable = (exceptionTable != null) ? exceptionTable : CodeException.EMPTY_CODE_EXCEPTION_ARRAY;
    this.attributes = (attributes != null) ? attributes : EMPTY_ARRAY;
    setLength(calculateLength());
  }
  
  public void accept(Visitor v) {
    v.visitCode(this);
  }
  
  private int calculateLength() {
    int len = 0;
    if (this.attributes != null)
      for (Attribute attribute : this.attributes)
        len += attribute.getLength() + 6;  
    return len + getInternalLength();
  }
  
  public Attribute copy(ConstantPool constantPool) {
    org.apache.bcel.classfile.Code c = (org.apache.bcel.classfile.Code)clone();
    if (this.code != null)
      c.code = (byte[])this.code.clone(); 
    c.setConstantPool(constantPool);
    c.exceptionTable = new CodeException[this.exceptionTable.length];
    Arrays.setAll(c.exceptionTable, i -> this.exceptionTable[i].copy());
    c.attributes = new Attribute[this.attributes.length];
    Arrays.setAll(c.attributes, i -> this.attributes[i].copy(constantPool));
    return c;
  }
  
  public void dump(DataOutputStream file) throws IOException {
    super.dump(file);
    file.writeShort(this.maxStack);
    file.writeShort(this.maxLocals);
    file.writeInt(this.code.length);
    file.write(this.code, 0, this.code.length);
    file.writeShort(this.exceptionTable.length);
    for (CodeException exception : this.exceptionTable)
      exception.dump(file); 
    file.writeShort(this.attributes.length);
    for (Attribute attribute : this.attributes)
      attribute.dump(file); 
  }
  
  public Attribute[] getAttributes() {
    return this.attributes;
  }
  
  public byte[] getCode() {
    return this.code;
  }
  
  public CodeException[] getExceptionTable() {
    return this.exceptionTable;
  }
  
  private int getInternalLength() {
    return 8 + this.code.length + 2 + 8 * ((this.exceptionTable == null) ? 0 : this.exceptionTable.length) + 2;
  }
  
  public LineNumberTable getLineNumberTable() {
    for (Attribute attribute : this.attributes) {
      if (attribute instanceof LineNumberTable)
        return (LineNumberTable)attribute; 
    } 
    return null;
  }
  
  public LocalVariableTable getLocalVariableTable() {
    for (Attribute attribute : this.attributes) {
      if (attribute instanceof LocalVariableTable)
        return (LocalVariableTable)attribute; 
    } 
    return null;
  }
  
  public int getMaxLocals() {
    return this.maxLocals;
  }
  
  public int getMaxStack() {
    return this.maxStack;
  }
  
  public void setAttributes(Attribute[] attributes) {
    this.attributes = (attributes != null) ? attributes : EMPTY_ARRAY;
    setLength(calculateLength());
  }
  
  public void setCode(byte[] code) {
    this.code = (code != null) ? code : ArrayUtils.EMPTY_BYTE_ARRAY;
    setLength(calculateLength());
  }
  
  public void setExceptionTable(CodeException[] exceptionTable) {
    this.exceptionTable = (exceptionTable != null) ? exceptionTable : CodeException.EMPTY_CODE_EXCEPTION_ARRAY;
    setLength(calculateLength());
  }
  
  public void setMaxLocals(int maxLocals) {
    this.maxLocals = maxLocals;
  }
  
  public void setMaxStack(int maxStack) {
    this.maxStack = maxStack;
  }
  
  public String toString() {
    return toString(true);
  }
  
  public String toString(boolean verbose) {
    StringBuilder buf = new StringBuilder(100);
    buf.append("Code(maxStack = ").append(this.maxStack).append(", maxLocals = ").append(this.maxLocals).append(", code_length = ").append(this.code.length).append(")\n")
      .append(Utility.codeToString(this.code, getConstantPool(), 0, -1, verbose));
    if (this.exceptionTable.length > 0) {
      buf.append("\nException handler(s) = \n").append("From\tTo\tHandler\tType\n");
      for (CodeException exception : this.exceptionTable)
        buf.append(exception.toString(getConstantPool(), verbose)).append("\n"); 
    } 
    if (this.attributes.length > 0) {
      buf.append("\nAttribute(s) = ");
      for (Attribute attribute : this.attributes) {
        buf.append("\n").append(attribute.getName()).append(":");
        buf.append("\n").append(attribute);
      } 
    } 
    return buf.toString();
  }
}
